Slender = {}
Slender.img = image("gfx/starkkz/slender.png", 0, 0, 1)

-- Variables
Slender.x, Slender.y, Slender.rot = 0, 0, 0

Slender.lastAttack = {}
Slender.isSeek = {}
Slender.screen = {}
Slender.screenAlpha = {}

Slender.visibleMode = true

Slender.cappearDuration = 0
Slender.cappearDelay = 0
Slender.lastAppear = 0
Slender.lastSpawn = 0

-- Pages
pages_found = 0
map_pages = 0

-- Config
Slender.killDist = 380
Slender.appearDuration = 5000
Slender.appearDelay = {25000,60000}

function Millisecs()
	return os.clock() * 1000
end

function FixRad(r)
	while r < -math.pi do r = r + math.pi*2 end
	while r > math.pi do r = r - math.pi*2 end
	return r
end

function RadBetween(fromx,fromy,tox,toy)
	return FixRad(math.atan2(fromy-toy,fromx-tox) - math.pi/2)
end

function AngleBetween(fromx,fromy,tox,toy)
	return math.deg(RadBetween(fromx, fromy, tox, toy))
end

function DistBetween(fx,fy,tx,ty)
	return math.sqrt((fx-tx)^2 + (fy-ty)^2)
end

function AngleInArea(x,y,x2,y2,angle,arc)
	local A = AngleBetween(x,y,x2,y2)
	return math.abs(A - angle) < arc/2
end

function Freeline(a,b,x,y)
	local Angle = RadBetween(a,b,x,y)
	local Dist = DistBetween(a,b,x,y)
	local _x = a + math.sin(Angle) * 10
	local _y = b - math.cos(Angle) * 10

	for i = 1, math.floor(Dist/10) do
		_x = _x + math.sin(Angle) * 10
		_y = _y - math.cos(Angle) * 10
		if not tile(math.floor(_x/32),math.floor(_y/32),"walkable") then
			return false, i*10
		end
	end
	return true
end

function LoadSlenderScreen()
	Slender.screen = {}
	Slender.screenAlpha = {}
	for i = 1, 32 do
		Slender.screen[i] = image("gfx/starkkz/slender_screen.jpg", 0, 0, 2, i)
		Slender.screenAlpha[i] = 0
		
		imagepos(Slender.screen[i], 320, 240, 0)
		imagealpha(Slender.screen[i], 0)
	end
end

function Slender:PlayerDist(id)
	return DistBetween(player(id,"x"),player(id,"y"),self.x,self.y)
end

function Slender:PlayerSees(id)
	if not Slender.visibleMode then return false end
	
	if AngleInArea(player(id,"x"), player(id,"y"), self.x, self.y, player(id,"rot"), 150) then
		if self:PlayerDist(id) < self.killDist then
			return Freeline(player(id,"x"),player(id,"y"),self.x,self.y)
		end
	end
end

function Slender:ChooseRandomTarget()
	local targets = player(0,"tableliving")
	if #targets > 0 then
		return targets[math.random(1,#targets)]
	end
end

function Slender:PlayerSpawnpoints(id)
	local spawnpoints = {}
	local x, y = player(id,"tilex"), player(id,"tiley")
	
	for _x = -3, 3 do
		for _y = -3, 3 do
			if math.abs(_x) + math.abs(_y) > 1 then
				if tile(x+_x,y+_y,"walkable") then
					if Freeline((x+_x)*32 + 16, (y+_y)*32 + 16, player(id,"x"), player(id,"y")) then
						table.insert(spawnpoints, {x+_x,y+_y})
					end
				end
			end
		end
	end
	return spawnpoints
end

function Slender:EnableVisibility()
	if #player(0,"table") > 0 then
		local target = self:ChooseRandomTarget()
		local spawnpoints = self:PlayerSpawnpoints(target)
	
		if #spawnpoints > 0 then
			local spawnpoint = spawnpoints[math.random(1,#spawnpoints)]
			local x, y = spawnpoint[1]*32 + 16, spawnpoint[2]*32 + 16
			
			self.x = x
			self.y = y
			self.rot = AngleBetween(self.x,self.y,player(target,"x"),player(target,"y"))
			self.visibleMode = true
			self.isSeek = {}
			self.lastSpawn = Millisecs()
			
			imagealpha(self.img, 1)
			imagepos(self.img, self.x, self.y, self.rot)
		end
	end
end

function Slender:DisableVisibility()
	self.visibleMode = false
	self.lastAppear = Millisecs()
	self.isSeek = {}
	imagealpha(self.img, 0)
end

addhook("join","onJoin")
function onJoin(id)
	Slender.isSeek[id] = false
end

addhook("startround","onRestart")
function onRestart()
	Slender.img = image("gfx/starkkz/slender.png", 0, 0, 1)
	imagealpha(Slender.img, 0)
	
	Slender.isSeek = {}
	Slender.lastAppear = 0
	Slender.lastSpawn = 0
	Slender.cappearDelay = 0
	Slender.cappearDuration = 0
	Slender.visibleMode = false
	
	pages_found = 0
	ReloadMapPages()
	LoadSlenderScreen()
	
	msg("Find all "..map_pages.." pages@C")
end

addhook("hit","onHit")
function onHit(id)
	return 1
end

addhook("always","Always")
function Always()
	if Slender.visibleMode then
		for _, id in pairs(player(0,"table")) do
			if not Slender.isSeek[id] and Slender:PlayerSees(id) then
				Slender.isSeek[id] = true
				parse("sv_sound2 "..id.." starkkz/slender_seek.ogg")
			end
		end
		
		if Millisecs() - Slender.lastSpawn >= Slender.appearDuration then
			local playerInvisible = true
			for _, id in pairs(player(0,"table")) do
				if math.abs(player(id,"x")-Slender.x) < 380 and math.abs(player(id,"y")-Slender.y) < 300 then
					playerInvisible = false
				end
			end
			if playerInvisible then
				Slender:DisableVisibility()
			end
		end
	else
		if pages_found > 0 then
			local appear_percent = pages_found/map_pages
			Slender.cappearDuration = Slender.appearDuration * appear_percent
			Slender.cappearDelay = math.random(Slender.appearDelay[1], Slender.appearDelay[2]) * (1 - (appear_percent*0.8))
		else
			if Slender.cappearDelay > 0 then Slender.cappearDelay = 0 end
			if Slender.cappearDuration > 0 then Slender.cappearDuration = 0 end
		end
		
		if Slender.cappearDelay > 0 then
			if Millisecs() - Slender.lastAppear >= Slender.cappearDelay then
				Slender:EnableVisibility()
			end
		end
	end
	
	for _, id in pairs(player(0,"table")) do
		if Slender.isSeek[id] and player(id,"health") > 0 and Slender:PlayerDist(id) < Slender.killDist and Freeline(Slender.x, Slender.y, player(id,"x"), player(id,"y")) then
			if Millisecs() - (Slender.lastAttack[id] or 0) >= 35 then
				parse("sethealth "..id.." "..player(id,"health")-math.random(2,3))
				UpdateSlenderScreenAlpha(id)
				Slender.lastAttack[id] = Millisecs()
				
				if player(id,"health") <= 0 then
					parse("sv_sound2 "..id.." starkkz/slender_kill.ogg")
				end
			end
		elseif player(id,"health") < player(id,"maxhealth") then
			parse("sethealth "..id.." "..player(id,"health")+1)
			UpdateSlenderScreenAlpha(id)
		end
	end
end

function UpdateSlenderScreenAlpha(id)
	if player(id,"health") > 0 then
		local a = 1 - player(id,"health")/player(id,"maxhealth")
		if Slender.screenAlpha[id] ~= a then
			imagealpha(Slender.screen[id], a)
			Slender.screenAlpha[id] = a
		end
	else
		if Slender.screenAlpha[id] ~= 0 then
			imagealpha(Slender.screen[id], 0)
			Slender.screenAlpha[id] = 0
		end
	end
end

Map_Page = {}
function CollectPage(id)
	if pages_found == 0 then
		Slender.lastAppear = Millisecs()
		Slender.lastSpawn = Millisecs()
	end

	pages_found = pages_found + 1
	for _, p in pairs(player(0,"table")) do
		if pages_found < map_pages then
			if p == id then
				msg2(p, "You've collected "..pages_found.."/"..map_pages.." pages!@C")
			else
				msg2(p, "We've collected "..pages_found.."/"..map_pages.." pages!@C")
			end
		elseif pages_found == map_pages then
			if p == id then
				msg2(p, "You've collected all pages!@C")
			else
				msg2(p, "We've collected all pages!@C")
			end
		end
	end
	
	if pages_found == map_pages then
		parse("sv_restart 5")
	end
end

timer(3000,"SlenderAmbient","",0)
function SlenderAmbient()
	if pages_found > 0 then
		if pages_found/map_pages < 0.5 then
			parse("sv_sound starkkz/slender_calm.ogg")
		else
			parse("sv_sound starkkz/slender_calm2.ogg")
		end
	end
end

addhook("use","onUse")
function onUse(id,event)
	for k, page in pairs(Map_Page) do
		if player(id,"tilex") == Map_Page[k].x and player(id,"tiley") == Map_Page[k].y then
			freeimage(Map_Page[k].img)
			Map_Page[k] = nil
			CollectPage(id)
			break
		end
	end
end

addhook("spawn","onSpawn")
function onSpawn(id)
	parse("setmaxhealth "..id.." 150")
	parse("speedmod "..id.." -5")
	return 0
end

function ReloadMapPages()
	map_pages = 0
	Map_Page = {}
	
	local e = entitylist()
	for k, v in pairs(e) do
		if entity(v.x,v.y,"name") == "page" then
			local img = image("gfx/starkkz/page.png",0,1,0)
			imagepos(img, v.x*32 + 16, v.y*32 + 16, 0)
			table.insert(Map_Page, {x = v.x,y = v.y,img = img})
			map_pages = map_pages + 1
		end
	end
end

ReloadMapPages()
LoadSlenderScreen()